#include "global.h"
#include "btstack/debug.h"
#include "1wire.h"

static unsigned char ds18b20_reg[10];


static const unsigned char crc88540_table[256] = { 
    0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65, 
  157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220, 
   35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98, 
  190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255, 
   70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7, 
  219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154, 
  101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36, 
  248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185, 
  140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205, 
   17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80, 
  175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238, 
   50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115, 
  202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139, 
   87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22, 
  233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168, 
  116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53 
}; 

static unsigned char get_crc(unsigned char *data, unsigned char count) 
{ 
	unsigned char result=0;  
	
	while(count--) { 
		result = crc88540_table[result ^ *data++]; 
	} 

	return result; 
}

/**********************************************************************
* Function:        unsigned char Detect_Slave_Device(void)
* PreCondition:    None
* Input:		   None	
* Output:		   1 - Not Present   0 - Present 	
* Overview:   	   To check the presence of slave device.    
***********************************************************************/
static unsigned char Detect_Slave_Device(void)
{
	if (!OW_reset_pulse())
		return HIGH;
	else 		
		return LOW;
}	

void ds18b20_convert(void)
{
	Detect_Slave_Device();

	OW_write_byte(DS18B20_SKIP_ROM);            // Send a command to read a serial number
	OW_write_byte(DS18B20_CONVERT_T);
}


//-----------------------------------------------------------------------------
// Obtain a celsius temperature value from a DS18B20 device.
//
int ds18b20_read_temp100_long(long *ptemp)
{
	// DS18B20 / DS1822 temperature register format :
	// LS Byte:     b7              b6              b5              b4              b3              b2              b1              b0
	//              2^3             2^2             2^1             2^0             2^-1    2^-2    2^-3    2^-4
	//
	// MS Byte:     b15             b14             b13             b12             b11             b10             b9              b8
	//              S               S               S               S               S               2^6             2^5             2^4
	
	int			k;
	long		temp;
	UINT16		crc;
	
	OW_reset_pulse();
	
	OW_write_byte(DS18B20_SKIP_ROM);				// skip ROM
	OW_write_byte(DS18B20_READ_SCRATCHPAD);			// read scratch pad
	
	for (k=0; k<9; k++){							// read 9 bytes from scratchpad
		ds18b20_reg[k]=OW_read_byte();
	}

	//check crc
	crc=get_crc(ds18b20_reg,8);
	if(crc!=ds18b20_reg[8]){
		return -1;
	}

	temp=(long)((short)((((UINT16)ds18b20_reg[1])<<8)|((UINT16)ds18b20_reg[0])));
	temp=(temp*100)/16;
	*ptemp=temp;

	return 0;
}

int init_ds18b20(void)
{
	if(!Detect_Slave_Device()){
		log_info("ds18b20 not detected\n");
		return -1;
	}
	log_info("ds18b20 detected\n");

	return 0;
}
