#include "btstack/debug.h"
#include "btstack/linked_list.h"
#include "btstack/remote_device_db.h"

#define SECTOR_START 0x1d07f000
#define	ROW_SIZE	128
#define	REC_MAX		7

extern linked_list_t db_mem_link_keys;

unsigned int NVMUnlock (unsigned int nvmop)
{
	unsigned int status;

	// Suspend or Disable all Interrupts
	asm volatile ("di %0" : "=r" (status));

	// Enable Flash Write/Erase Operations and Select
	// Flash operation to perform
	NVMCON = nvmop;

	// Write Keys
	NVMKEY = 0xAA996655;
	NVMKEY = 0x556699AA;

	// Start the operation using the Set Register
	NVMCONSET = 0x8000;

	// Wait for operation to complete
	while (NVMCON & 0x8000);

	// Restore Interrupts
	if (status & 0x00000001)
		asm volatile ("ei");
	else
		asm volatile ("di");

	// Disable NVM write enable
	NVMCONCLR = 0x0004000;

	// Return WRERR and LVDERR Error Status Bits 
	return (NVMCON & 0x3000);
}

unsigned int NVMErasePage(void* address)
{
	unsigned int res;

	// Set NVMADDR to the Start Address of page to erase
	NVMADDR = (unsigned int) address;

	// Unlock and Erase Page
	res = NVMUnlock(0x4004);

	// Return Result
	return res;
}

unsigned int NVMWriteWord (void* address, unsigned int data)
{
	unsigned int res;

	// Load data into NVMDATA register
	NVMDATA = data;

	 // Load address to program into NVMADDR register
	NVMADDR = (unsigned int) address;

	// Unlock and Write Word
	res = NVMUnlock (0x4001);

	// Return Result
	return res;
}


void    erase_flashmem(unsigned long progAddr)
{
	NVMErasePage((void*)progAddr);
	return;
}

void prog_flashmem_row2(unsigned long progAddr,BYTE* progData1,size_t len1,BYTE* progData2,size_t len2)
{
	DWORD	src;
	size_t	len;

	len1=(len1+3)/4;
	for(len=0;len<len1;len++){
		memcpy(&src,progData1,4);
		progData1+=4;
		NVMWriteWord((void*)progAddr,src);
		progAddr+=4;
	}

	len2=(len2+3)/4;
	for(len=0;len<len2;len++){
		memcpy(&src,progData2,4);
		progData2+=4;
		NVMWriteWord((void*)progAddr,src);
		progAddr+=4;
	}

}

void read_flashmem(unsigned long progAddr,BYTE* progData,size_t len)
{
	memcpy(progData,(void*)(0x80000000LU+progAddr),len);
}

void save_link_keys(linked_list_t list)
{
	DWORD	num=0;
    unsigned long sec=SECTOR_START;

    erase_flashmem(sec);
    sec+=ROW_SIZE;

	linked_item_t *it;
    for (it = (linked_item_t *) list; it && num<REC_MAX ; it = it->next){
        db_mem_device_link_key_t * item = (db_mem_device_link_key_t *) it;
		prog_flashmem_row2(sec,((BYTE *)&item->device.bd_addr),BD_ADDR_LEN,((BYTE *)item->link_key),LINK_KEY_LEN);
		sec+=ROW_SIZE;
		num++;
    }
	prog_flashmem_row2(SECTOR_START,(BYTE*)&num,sizeof(num),NULL,0);
	log_info("save_link_keys: %u keys\r\n",num);
}

void load_link_keys(remote_device_db_t * lpremote_device_db)
{
	int	num;
	DWORD	num_max;
    unsigned long sec=SECTOR_START;
	db_mem_device_link_key_t item;

    read_flashmem(sec,(BYTE*)&num_max,sizeof(num_max));
    if(num_max>REC_MAX){
        //error
		log_info("load_link_keys: invalid num_max:%u\r\n",num_max);
        return;
    }
	sec+=ROW_SIZE;
	for(num=0;num<num_max;num++){
        read_flashmem(sec+0,((BYTE *)&item.device.bd_addr),BD_ADDR_LEN);
        read_flashmem(sec+(((BD_ADDR_LEN+3)/4)*4),((BYTE *)item.link_key),LINK_KEY_LEN);
		lpremote_device_db->put_link_key(&item.device.bd_addr, &item.link_key);
		sec+=ROW_SIZE;
	}
	log_info("load_link_keys: %u keys\r\n",num);
}

void print_link_keys(linked_list_t list)
{
#ifdef	__DEBUG
    linked_item_t *it;
    for (it = (linked_item_t *) list; it ; it = it->next){
        db_mem_device_link_key_t * item = (db_mem_device_link_key_t *) it;

		log_info("bd-addr:%s  key:%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x\n"
			,bd_addr_to_str(item->device.bd_addr)
			,item->link_key[0]
			,item->link_key[1]
			,item->link_key[2]
			,item->link_key[3]
			,item->link_key[4]
			,item->link_key[5]
			,item->link_key[6]
			,item->link_key[7]
			,item->link_key[8]
			,item->link_key[9]
			,item->link_key[10]
			,item->link_key[11]
			,item->link_key[12]
			,item->link_key[13]
			,item->link_key[14]
			,item->link_key[15]
		);
	}
#endif
}
