#ifndef __GLOBAL_H__
#define __GLOBAL_H__

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_bluetooth.h"
#include "user.h"
#include "xprintf.h"

#define PIN_CODE_DEFAULT	"0000"
#define BOARD_SBDBT32       1

// Define the baud rate constants UART1
#define	UARTPARAM1			UARTPARAM_N81			//data=8bit parity=none stopbit=1bit

#define BAUDRATE1       	115200
#define	USE_RFCOMM_BAUDRATE	1						//0:baud rate as fixed 1:baud rate as RFCOMM parameter from Master
#define BRG_DIV1        	4
#define BRGH1           	1
// Define Hardware Flow Control UART1 (0:off 1:on)
#define	UART1_HW_FLOW		0
#define	UART1_TX_POLARITY	0						//MMɐ 0:ʏ 1:]
#define	UART1_RX_POLARITY	0						//MMɐ 0:ʏ 1:]

// Define the baud rate constants UART2 (Debug)
#define BAUDRATE2       	115200
#define BRG_DIV2        	4
#define BRGH2           	1

// Define LED1 communication indicator on time(ms)
#define	LED1_ON_COUNT		10
#define	USE_RSSI			1						//0:blink rate as fixed 1:blink rate as RSSI (fast:lowRSSI slow:highRSSI)



#if BOARD_SBDBT32
#define LOCAL_NAME			"SBDBT32"
#define	ICSDEF				ICS_PGx1
#endif

#ifdef	__DEBUG
//disable watchdog timer
#define	FWDTENDEF			FWDTEN_OFF
#define	RestartWatchdog()
#define STO_SERIAL			1   //0:STO as I/O  1:STO as serial output
#else
//enable watchdog timer
#define	FWDTENDEF			FWDTEN_ON
#define	RestartWatchdog()	ClrWdt()
#define STO_SERIAL			0   //0:STO as I/O  1:STO as serial output
#endif

#if BOARD_SBDBT32

//
//for SBDBT32 board
//
// Configure LED1 - put on pin 14 (RB14)
#define	led1_setup()		(TRISBbits.TRISB14=0)
#define	led1_on()			(LATBbits.LATB14=0)	//ON
#define	led1_off()			(LATBbits.LATB14=1)	//OFF

// Configure LED2 - put on pin 15 (RB15)
#define	led2_setup()		(TRISBbits.TRISB15=0)
#define	led2_on()			(LATBbits.LATB15=0)	//ON
#define	led2_off()			(LATBbits.LATB15=1)	//OFF

#define	HIGH	1
#define	LOW		0
#define	OUTPUT	0
#define	INPUT 	1
#define	SET		1
#define	CLEAR	0

#define DELAY_6Us	6
#define DELAY_9Us	9
#define DELAY_10Us	10
#define DELAY_55Us	55
#define DELAY_60Us	60
#define DELAY_64Us	64
#define DELAY_70Us	70
#define DELAY_205Us	205			// DELAY_410Us = DELAY_205Us + DELAY_205Us
#define DELAY_240Us	240			// DELAY_480Us = DELAY_240Us + DELAY_240Us // since the variable is declared
//#define	wait(x)		Delay10us(x/27)
#define	wait(x)		Delay10us(x/5)

#define DS18B20_SKIP_ROM				0xCC
#define DS18B20_CONVERT_T				0x44
#define DS18B20_READ_ROM				0x33
#define DS18B20_READ_SCRATCHPAD			0xBE

typedef unsigned long IPLSTAT;
#define	GetIPL()			_CP0_GET_STATUS()
#define	DisableInterrupt()	_CP0_SET_STATUS(_CP0_GET_STATUS()|(0x3fUL<<10))
#define	EnableInterrupt(x)	(_CP0_SET_STATUS(x))


//1wire port:SBDBT32 PIN9
#define	OW_setup()			(TRISDbits.TRISD9=1,ODCDbits.ODCD9=1)	//input,open-drain
#define OW_PIN_DIRECTION 	TRISDbits.TRISD9
#define OW_WRITE_PIN  		LATDbits.LATD9
#define OW_READ_PIN			PORTDbits.RD9

#if	STO_SERIAL
#define	sto_setup()
#define	sto_l()
#define	sto_h()
#else
#define	sto_setup()
#define	sto_l()
#define	sto_h()
#endif

// no association out on this board
#define	assoc_setup()
#define	assoc_l()
#define	assoc_h()

// no on out on this board
#define	on_setup()

#define	pairen_setup()
#define	get_pairen()		(1)

#endif


extern void (*usb_tick_handler)(void);
void	event_bluetooth_rxEvent_done(WORD size);
void	event_bluetooth_rxAcl_done(WORD size);

int init_ds18b20(void);
void ds18b20_convert(void);
int ds18b20_read_temp100_long(long *ptemp);
#endif
